# Plugin Triggering
POD has functionality to notify running Plugins upon new item arrival.
The flow looks as follows:
![diagram](plugin_triggering.png)

## Trigger registration request body
In order to have Plugin notified, trigger registration request has to be send. Here is an example of it
```
POST /v4/bulk
```
```json
{
  "auth": {
        ...
  },
  "payload": {
    "createItems": [
      {
        "type": "Trigger",
        "id": "fffdd970-06f8-490d-94d9-456df610bc15",
        "dateCreated": null,
        "dateModified": null,
        "dateServerModified": null,
        "deleted": false,

        "action": "v1/item/trigger",
        "triggerOn": "Message",
        "pluginRunId": "4fa2094c-51d6-4874-a135-9017fcdedf16",
        "filterCreatedAfter": 1655170839992
      },

      {
        "type": "Edge",
        "id": "a4a42fa1-ef3d-4907-81c9-aff0ffc582ce",
        "dateCreated": null,
        "dateModified": null,
        "dateServerModified": null,
        "deleted": false
      }
    ],
    "updateItems": [],
    "deleteItems": [],
    "createEdges": [
      {
        "_source": "fffdd970-06f8-490d-94d9-456df610bc15",
        "_target": "4fa2094c-51d6-4874-a135-9017fcdedf16",
        "_self": "a4a42fa1-ef3d-4907-81c9-aff0ffc582ce",
        "_name": "trigger"
      }
    ]
  }
}
```

The request creates ```Trigger``` object that holds information what ```action``` has to be taken upon new item arrival (which plugin endpoint to reach). The ```triggerOn``` defines what items are in the interest, it will trigger only for them. Filed ```filterCreatedAfter``` defines timestamp of items that should fire the trigger. Only those created after it gonna be used.

The ```action``` property currently can be set only to one value, ```/v1/item/trigger```. That's the endpoint used by POD to do the POST request with following body:
```
{
  item_id: string,
  trigger_id: string
}

```

Where ```item_id```, is the id of newly created item, and ```trigger_id``` identifies trigger instance.

## Plugin status notification to the POD
`Plugin` sends the status of the processing to the `POD` upon completion by POST request on endpoint ```v4/trigger/status```. The body contains

```
{
  item_id: string,
  trigger_id: string,
  status: string
}
```
## Frontend status notification
The frontend part currently uses polling to get the item status.

## The `trigger` edge
Another object in the reqest is the Edge of type ```trigger``` that connects the ```Trigger``` object with ```PluginRun``` instance.

There can be many Triggers per Plugin but only one Plugin for Trigger.

## Deregistering the Trigger
It's possible to unregister the Trigger, simply by removing it from the database.