use criterion::{criterion_group, criterion_main, Criterion};
use libpod::{
    database_api, database_migrate_refinery,
    internal_api::{graphql, search},
};

mod common;
mod test_dataset;
use serde_json::json;

#[cfg(test)]
pub fn deep_join(c: &mut Criterion) {
    let mut conn = rusqlite::Connection::open_in_memory().unwrap();
    conn.set_prepared_statement_cache_capacity(0);

    // can this be done in more elegant way?
    {
        let tx = conn.transaction().unwrap();
        let _res = tx.execute("PRAGMA cache_size = 0;", []);
        let _ = tx.commit();
    }

    database_migrate_refinery::embedded::migrations::runner()
        .run(&mut conn)
        .expect("Failed to run refinery migrations");

    let tx = conn.transaction().unwrap();

    let cli = common::default_cli();
    let pod_owner = "";

    test_dataset::prepare_graphql_dataset(&tx, pod_owner, cli);

    let schema = database_api::get_schema(&tx).unwrap();

    // search
    let search_json = json!({
        "type": "Person1",
        "[[edges]]": {}
    });

    let res = search(
        &tx,
        &schema,
        serde_json::from_value(search_json.clone()).unwrap(),
    )
    .unwrap();
    assert_eq!(res.len(), 1000);

    // graphql search
    let query = "
    query {
        Person1 {
           id
           friend1 {
               id
                friend2 {
                    id
                }
           }
        }
    }"
    .to_owned();

    let _res = graphql(&tx, &schema, query.clone());

    // print!("{:}", serde_json::to_string_pretty(&json!(_res.unwrap())).unwrap());

    c.bench_function("search api", |b| {
        b.iter(|| {
            search(
                &tx,
                &schema,
                serde_json::from_value(search_json.clone()).unwrap(),
            )
            .unwrap()
        })
    });

    c.bench_function("graphql deep join", |b| {
        b.iter(|| graphql(&tx, &schema, query.clone()).unwrap())
    });
}

criterion_group!(benches, deep_join);
criterion_main!(benches);
