mod warp_api;
mod warp_endpoints;

use libpod::internal_api::get_project_version;
use libpod::{
    command_line_interface::{self, CliOptions},
    constants,
};
use std::{fs::create_dir_all, path::PathBuf, str::FromStr};
use tracing::{error, info};
use tracing_subscriber::{EnvFilter, FmtSubscriber};

#[tokio::main]
async fn main() {
    let builder = FmtSubscriber::builder()
        .with_env_filter(EnvFilter::from_default_env())
        .with_filter_reloading();

    let handle = builder.reload_handle();

    let subscriber = builder.finish();

    tracing::subscriber::set_global_default(subscriber).expect("setting default subscriber failed");

    let cli_options: CliOptions = command_line_interface::PARSED.clone();
    info!("Starting Pod version {}", get_project_version());
    info!("Running Pod with configuration {:#?}", cli_options);

    create_config_directory(constants::DATABASE_DIR);
    create_config_directory(constants::FILES_DIR);

    // Start web framework
    warp_api::run_server(cli_options, handle).await;
}

fn create_config_directory(path: &str) {
    if let Ok(path) = PathBuf::from_str(path) {
        create_dir_all(path).expect("Failed to create database directory");
    } else {
        error!("Failed to parse database directory {:?}", path);
        std::process::exit(1)
    };
}
